@@ -1,3 +1,5 @@
+//! Install launchers like Web Application from your browser or Steam.
+//!
 //! # Examples
 //!
 //! ```rust,no_run
@@ -22,7 +24,7 @@
 //!     let icon = Icon::Bytes(buffer);
 //!     let response = proxy
 //!         .prepare_install(
-//!             &WindowIdentifier::default(),
+//!             None,
 //!             "SomeApp",
 //!             icon,
 //!             PrepareInstallOptions::default()
@@ -56,7 +58,7 @@ use serde_repr::{Deserialize_repr, Serialize_repr};
 use zbus::zvariant::{self, DeserializeDict, OwnedValue, SerializeDict, Type, Value};
 
 use super::{HandleToken, Icon, Request};
-use crate::{proxy::Proxy, Error, WindowIdentifier};
+use crate::{proxy::Proxy, ActivationToken, Error, WindowIdentifier};
 
 #[bitflags]
 #[derive(Default, Serialize_repr, Deserialize_repr, PartialEq, Eq, Debug, Copy, Clone, Type)]
@@ -73,6 +75,8 @@ pub enum LauncherType {
     WebApplication,
 }
 
+#[cfg_attr(feature = "glib", derive(glib::Enum))]
+#[cfg_attr(feature = "glib", enum_type(name = "AshpdIconType"))]
 #[derive(Debug, Clone, Copy, Serialize, Deserialize, PartialEq, Eq, PartialOrd, Ord, Type)]
 #[zvariant(signature = "s")]
 #[serde(rename_all = "lowercase")]
@@ -190,6 +194,25 @@ impl std::fmt::Debug for PrepareInstallResponse {
     }
 }
 
+#[derive(SerializeDict, Type, Debug, Default)]
+#[zvariant(signature = "dict")]
+/// Options to pass to [`DynamicLauncherProxy::launch`]
+pub struct LaunchOptions {
+    activation_token: Option<ActivationToken>,
+}
+
+impl LaunchOptions {
+    /// Sets the token that can be used to activate the chosen application.
+    #[must_use]
+    pub fn activation_token(
+        mut self,
+        activation_token: impl Into<Option<ActivationToken>>,
+    ) -> Self {
+        self.activation_token = activation_token.into();
+        self
+    }
+}
+
 #[derive(Debug)]
 /// Wrong type of [`crate::desktop::Icon`] was used.
 pub struct UnexpectedIconError;
@@ -204,7 +227,7 @@ impl std::fmt::Display for UnexpectedIconError {
 /// The interface lets sandboxed applications install launchers like Web
 /// Application from your browser or Steam.
 ///
-/// Wrapper of the DBus interface: [`org.freedesktop.portal.DynamicLauncher`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-org.freedesktop.portal.DynamicLauncher).
+/// Wrapper of the DBus interface: [`org.freedesktop.portal.DynamicLauncher`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html).
 #[derive(Debug)]
 #[doc(alias = "org.freedesktop.portal.DynamicLauncher")]
 pub struct DynamicLauncherProxy<'a>(Proxy<'a>);
@@ -220,13 +243,13 @@ impl<'a> DynamicLauncherProxy<'a> {
     ///
     ///  # Specifications
     ///
-    /// See also [`PrepareInstall`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.PrepareInstall).
+    /// See also [`PrepareInstall`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-prepareinstall).
     #[doc(alias = "PrepareInstall")]
     #[doc(alias = "xdp_portal_dynamic_launcher_prepare_install")]
     #[doc(alias = "xdp_portal_dynamic_launcher_prepare_install_finish")]
     pub async fn prepare_install(
         &self,
-        parent_window: &WindowIdentifier,
+        identifier: Option<&WindowIdentifier>,
         name: &str,
         icon: Icon,
         options: PrepareInstallOptions,
@@ -234,12 +257,12 @@ impl<'a> DynamicLauncherProxy<'a> {
         if !icon.is_bytes() {
             return Err(UnexpectedIconError {}.into());
         }
-
+        let identifier = identifier.map(|i| i.to_string()).unwrap_or_default();
         self.0
             .request(
                 &options.handle_token,
                 "PrepareInstall",
-                &(parent_window, name, icon.as_value(), &options),
+                &(identifier, name, icon.as_value(), &options),
             )
             .await
     }
@@ -248,7 +271,7 @@ impl<'a> DynamicLauncherProxy<'a> {
     ///
     /// # Specifications
     ///
-    /// See also [`RequestInstallToken`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.RequestInstallToken).
+    /// See also [`RequestInstallToken`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-requestinstalltoken).
     #[doc(alias = "RequestInstallToken")]
     #[doc(alias = "xdp_portal_dynamic_launcher_request_install_token")]
     pub async fn request_install_token(&self, name: &str, icon: Icon) -> Result<String, Error> {
@@ -265,7 +288,7 @@ impl<'a> DynamicLauncherProxy<'a> {
 
     /// # Specifications
     ///
-    /// See also [`Install`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.Install).
+    /// See also [`Install`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-install).
     #[doc(alias = "Install")]
     #[doc(alias = "xdp_portal_dynamic_launcher_install")]
     pub async fn install(
@@ -283,7 +306,7 @@ impl<'a> DynamicLauncherProxy<'a> {
 
     /// # Specifications
     ///
-    /// See also [`Uninstall`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.Uninstall).
+    /// See also [`Uninstall`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-uninstall).
     #[doc(alias = "Uninstall")]
     #[doc(alias = "xdp_portal_dynamic_launcher_uninstall")]
     pub async fn uninstall(&self, desktop_file_id: &str) -> Result<(), Error> {
@@ -296,7 +319,7 @@ impl<'a> DynamicLauncherProxy<'a> {
 
     /// # Specifications
     ///
-    /// See also [`GetDesktopEntry`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.GetDesktopEntry).
+    /// See also [`GetDesktopEntry`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-getdesktopentry).
     #[doc(alias = "GetDesktopEntry")]
     #[doc(alias = "xdp_portal_dynamic_launcher_get_desktop_entry")]
     pub async fn desktop_entry(&self, desktop_file_id: &str) -> Result<String, Error> {
@@ -305,7 +328,7 @@ impl<'a> DynamicLauncherProxy<'a> {
 
     /// # Specifications
     ///
-    /// See also [`GetIcon`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.GetIcon).
+    /// See also [`GetIcon`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-geticon).
     #[doc(alias = "GetIcon")]
     #[doc(alias = "xdp_portal_dynamic_launcher_get_icon")]
     pub async fn icon(&self, desktop_file_id: &str) -> Result<LauncherIcon, Error> {
@@ -314,18 +337,16 @@ impl<'a> DynamicLauncherProxy<'a> {
 
     /// # Specifications
     ///
-    /// See also [`Launch`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-DynamicLauncher.Launch).
+    /// See also [`Launch`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-launch).
     #[doc(alias = "Launch")]
     #[doc(alias = "xdp_portal_dynamic_launcher_launch")]
-    pub async fn launch(&self, desktop_file_id: &str) -> Result<(), Error> {
-        // TODO: handle activation_token
-        let options: HashMap<&str, zvariant::Value<'_>> = HashMap::new();
+    pub async fn launch(&self, desktop_file_id: &str, options: LaunchOptions) -> Result<(), Error> {
         self.0.call("Launch", &(desktop_file_id, &options)).await
     }
 
     /// # Specifications
     ///
-    /// See also [`SupportedLauncherTypes`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-property-org-freedesktop-portal-DynamicLauncher.SupportedLauncherTypes).
+    /// See also [`SupportedLauncherTypes`](https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.DynamicLauncher.html#org-freedesktop-portal-dynamiclauncher-supportedlaunchertypes).
     #[doc(alias = "SupportedLauncherTypes")]
     pub async fn supported_launcher_types(&self) -> Result<BitFlags<LauncherType>, Error> {
         self.0
@@ -334,14 +355,21 @@ impl<'a> DynamicLauncherProxy<'a> {
     }
 }
 
+impl<'a> std::ops::Deref for DynamicLauncherProxy<'a> {
+    type Target = zbus::Proxy<'a>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
 
     #[test]
     fn test_icon_signature() {
-        let signature = LauncherIcon::signature();
-        assert_eq!(signature.as_str(), "(vsu)");
+        assert_eq!(LauncherIcon::SIGNATURE, "(vsu)");
 
         let icon = vec![IconType::Png];
         assert_eq!(serde_json::to_string(&icon).unwrap(), "[\"png\"]");
